Index: adapted/adt/detail/extension.hpp
===================================================================
--- adapted/adt/detail/extension.hpp	(revision 78486)
+++ adapted/adt/detail/extension.hpp	(working copy)
@@ -12,13 +12,28 @@
 
 #include <boost/type_traits/remove_const.hpp>
 #include <boost/type_traits/remove_reference.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
+#include <boost/fusion/adapted/struct/detail/extension.hpp>
 
-namespace boost { namespace fusion { namespace detail
-{
-    template <typename T, typename Dummy>
-    struct get_identity
-      : remove_const<typename remove_reference<T>::type>
-    {};
-}}}
+namespace boost { namespace fusion
+{ 
+    namespace detail
+    {
+        template <typename T, typename Dummy>
+        struct get_identity
+          : remove_const<typename remove_reference<T>::type>
+        {};
+    }
+    
+    namespace extension
+    {
+        // Overload as_readonly() to unwrap adt_attribute_proxy.
+        template <typename T, int N, bool Const>
+        typename adt_attribute_proxy<T, N, Const>::type as_readonly(const adt_attribute_proxy<T, N, Const>& proxy)
+        {
+            return proxy.get();
+        }
+    }
+}}
 
 #endif
Index: support.hpp
===================================================================
--- support.hpp	(revision 78486)
+++ support.hpp	(working copy)
@@ -19,5 +19,6 @@
 #include <boost/fusion/support/deduce.hpp>
 #include <boost/fusion/support/deduce_sequence.hpp>
 #include <boost/fusion/support/unused.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 #endif
Index: sequence/comparison/detail/equal_to.hpp
===================================================================
--- sequence/comparison/detail/equal_to.hpp	(revision 78486)
+++ sequence/comparison/detail/equal_to.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,7 +33,7 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a == *b
+            return extension::as_readonly(*a) == extension::as_readonly(*b)
                 && call(fusion::next(a), fusion::next(b));
         }
 
Index: sequence/comparison/detail/greater.hpp
===================================================================
--- sequence/comparison/detail/greater.hpp	(revision 78486)
+++ sequence/comparison/detail/greater.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,8 +33,9 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a > *b ||
-                (!(*b > *a) && call(fusion::next(a), fusion::next(b)));
+            return extension::as_readonly(*a) > extension::as_readonly(*b) ||
+                (!(extension::as_readonly(*b) > extension::as_readonly(*a)) && 
+                 call(fusion::next(a), fusion::next(b)));
         }
 
         template <typename I1, typename I2>
Index: sequence/comparison/detail/not_equal_to.hpp
===================================================================
--- sequence/comparison/detail/not_equal_to.hpp	(revision 78486)
+++ sequence/comparison/detail/not_equal_to.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,7 +33,7 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a != *b
+            return extension::as_readonly(*a) != extension::as_readonly(*b)
                 || call(fusion::next(a), fusion::next(b));
         }
 
Index: sequence/comparison/detail/less_equal.hpp
===================================================================
--- sequence/comparison/detail/less_equal.hpp	(revision 78486)
+++ sequence/comparison/detail/less_equal.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,8 +33,9 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a <= *b
-                && (!(*b <= *a) || call(fusion::next(a), fusion::next(b)));
+            return extension::as_readonly(*a) <= extension::as_readonly(*b)
+                && (!(extension::as_readonly(*b) <= extension::as_readonly(*a)) || 
+                    call(fusion::next(a), fusion::next(b)));
         }
 
         template <typename I1, typename I2>
Index: sequence/comparison/detail/greater_equal.hpp
===================================================================
--- sequence/comparison/detail/greater_equal.hpp	(revision 78486)
+++ sequence/comparison/detail/greater_equal.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,8 +33,9 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a >= *b
-                && (!(*b >= *a) || call(fusion::next(a), fusion::next(b)));
+            return extension::as_readonly(*a) >= extension::as_readonly(*b)
+                && (!(extension::as_readonly(*b) >= extension::as_readonly(*a)) || 
+                    call(fusion::next(a), fusion::next(b)));
         }
 
         template <typename I1, typename I2>
Index: sequence/comparison/detail/less.hpp
===================================================================
--- sequence/comparison/detail/less.hpp	(revision 78486)
+++ sequence/comparison/detail/less.hpp	(working copy)
@@ -12,6 +12,7 @@
 #include <boost/fusion/iterator/deref.hpp>
 #include <boost/fusion/iterator/next.hpp>
 #include <boost/fusion/iterator/equal_to.hpp>
+#include <boost/fusion/support/as_readonly.hpp>
 
 namespace boost { namespace fusion { namespace detail
 {
@@ -32,8 +33,9 @@
         static bool
         call(I1 const& a, I2 const& b, mpl::false_)
         {
-            return *a < *b ||
-                (!(*b < *a) && call(fusion::next(a), fusion::next(b)));
+            return extension::as_readonly(*a) < extension::as_readonly(*b) ||
+                (!(extension::as_readonly(*b) < extension::as_readonly(*a)) && 
+                 call(fusion::next(a), fusion::next(b)));
         }
 
         template <typename I1, typename I2>
Index: support/as_readonly.hpp
===================================================================
--- support/as_readonly.hpp	(revision 0)
+++ support/as_readonly.hpp	(revision 0)
@@ -0,0 +1,27 @@
+/*=============================================================================
+    Copyright (c) 2012 Nathan Ridge
+
+    Distributed under the Boost Software License, Version 1.0. (See accompanying 
+    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+==============================================================================*/
+#ifndef BOOST_FUSION_SUPPORT_AS_READONLY_HPP
+#define BOOST_FUSION_SUPPORT_AS_READONLY_HPP
+
+namespace boost { namespace fusion { namespace extension
+{
+    
+    // A customization point that allows certain wrappers around
+    // Fusion sequence elements (e.g. adt_attribute_proxy) to be 
+    // unwrapped in contexts where the element only needs to be
+    // read. The library wraps accesses to Fusion elements in
+    // such contexts with calls to this function. Users can
+    // specialize this function for their own wrappers.
+    template <typename T>
+    const T& as_readonly(const T& obj)
+    {
+        return obj;
+    }
+    
+}}}
+
+#endif
